@@ -24,13 +24,16 @@ from matplotlib.patches import FancyArrowPatch
 from mpl_toolkits.mplot3d import proj3d
 
 # Package functions
+from epsproc.util.listFuncs import BLMdimList
 from epsproc.sphPlot import plotTypeSelector
 # from epsproc.util import matEleSelector  # Throws error, due to circular import refs?
+# from epsproc.util.conversion import multiDimXrToPD
+# from epsproc.util import matEdimList, BLMdimList, dataTypesList, multiDimXrToPD
 
 # Additional plotters
 try:
     import seaborn as sns
-    import epsproc._sns_matrixMod as snsMatMod  # SNS code with modified clustermap
+    import epsproc._sns_matrixMod as snsMatMod  # SNS code with modified clustermap - NOTE this may need Seaborn 0.9.0 - TBC
 except ImportError as e:
     if e.msg != "No module named 'seaborn'":
         raise
@@ -135,7 +138,7 @@ def molPlot(molInfo):
 
 #*** BLM surface plots
 
-def BLMplot(BLM, thres = 1e-2, thresType = 'abs', xDim = 'Eke', backend = 'xr'):
+def BLMplot(BLM, thres = 1e-2, thresType = 'abs', xDim = 'Eke', backend = 'xr', **kwargs):
     """
     Plotting routines for BLM values from Xarray.
     Plot line or surface plot, with various backends available.
@@ -158,8 +161,12 @@ def BLMplot(BLM, thres = 1e-2, thresType = 'abs', xDim = 'Eke', backend = 'xr'):
 
     backend : str, optional, default = 'xr'
         Plotter to use. Default is 'xr' for Xarray internal plotting. May be switched according to plot type in future...
+        **kwargs are currently passed to the plotter backend.
 
     """
+    # Local/deferred import to avoid circular import issues at module level.
+    # TO DO: Should fix with better __init__!
+    from epsproc.util import matEleSelector
 
     # Check dims, and set facet dim
     dims = BLMdimList()
@@ -171,7 +178,7 @@ def BLMplot(BLM, thres = 1e-2, thresType = 'abs', xDim = 'Eke', backend = 'xr'):
         thres = thres * BLM.max()
 
     # Threshold results. Set to check along Eke dim, but may want to pass this as an option.
-    BLMplot = ep.matEleSelector(BLM, thres=thres, dims = xDim)
+    BLMplot = matEleSelector(BLM, thres=thres, dims = xDim)
 
     # Set BLM index for plotting.
     # Necessary for Xarray plotter with multi-index categories it seems.
@@ -179,7 +186,7 @@ def BLMplot(BLM, thres = 1e-2, thresType = 'abs', xDim = 'Eke', backend = 'xr'):
 
     #*** Plot
     if backend is 'xr':
-        BLMplot.real.squeeze().plot(x=xDim, y='BLMind', col=cDims, size = 5)
+        BLMplot.real.squeeze().plot(x=xDim, y='BLMind', col=cDims, size = 5, **kwargs)
 
 
 
@@ -190,11 +197,12 @@ def symListGen(data):
     symList = []
     [symList.append(list(item)) for item in data.get_index('Sym').to_list()]
 
-    return np.ravel(symList)
+    return np.unique(np.ravel(symList))
 
 def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, logFlag = False, eulerGroup = True,
-        selDims = None, sumDims = None, plotDims = ('l','m','mu','Cont','Targ','Total','it','Type'), squeeze = True,
-        xDim = 'Eke', backend = 'sns', cmap = None, figsize = None, verbose = False):
+        selDims = None, sumDims = None, plotDims = None, squeeze = False, fillna = False,
+        xDim = 'Eke', backend = 'sns', cmap = None, figsize = None, verbose = False, mMax = 10, titleString = None,
+        labelRound = 3):
     """
     Plotting routine for ePS matrix elements & BLMs.
 
@@ -234,15 +242,26 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
     sumDims : tuple, optional, default = None
         Dimensions to sum over from the input Xarray.
 
-    plotDims : tuple, optional, default = ('l','m','mu','Cont','Targ','Total','it','Type')
+    plotDims : tuple, optional, default = None
         Dimensions to stack for plotting, also controls order of stacking (hence sorting and plotting).
+        Default case plots all dims not included in (xDim, sumDims, selDims).
+        E.g. for matrix elements this  will be ('l','m','mu','Cont','Targ','Total','it','Type')
         TO DO: auto generation for different dataType, also based on selDims and sumDims selections.
+        11/03/20: partially fixed, now add any missing dims to plot automatically.
+        NOTE: this currently fails for 'Labels' case, not sure why.
 
     squeeze : bool, optional, default = True
         Drop singleton dimensions from plot.
 
+    fillna : bool, optional, default = False
+        Fill NaN values with 0 if True.
+
     xDim : str, optional, default = 'Eke'
         Dimension to use for x-axis, also used for thresholding. Default plots (Eke, LM) surfaces.
+        NOTE: if xDim is a MultiIndex, pass as a dictionary mapping, otherwise it may be unstacked during data prep.
+        E.g. for plotting stacked (L,M), set xDim = {'LM':['L','M']}
+        NOTE: this is currently also passed to matEleSelector(), so stacked dims *must* exist in inital Xarray.
+        THIS SHOULD BE FIXED, since it's a pain.
 
     backend : str, optional, default = 'sns'
         Plotter to use. Default is 'sns' for Seaborn clustermap plot.
@@ -259,6 +278,18 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
     verbose : bool, optional, default False
         Print debug info.
 
+    mMax : int, optional, default = 10
+        Used as default for m colour mapping, in cases where coords are not parsed.
+        TODO: fix this, it's ugly.
+
+    titleString : str, optional, default = None
+        Set main title for plot. If not set, filename or data.attr['jobLabel'] will be used if present.
+        Some plot settings labels will also be appended to this.
+
+    labelRound : int, optional, default = 3
+        Round to N d.p. for plot labels (legend text).
+        This is also passed to :py:func:`epsproc.multiDimXrToPD()` for column labels (for E, t or Euler dims only)
+
     Returns
     -------
     daPlot : Xarray
@@ -276,6 +307,7 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
     * For clustermap use local version - code from Seaborn, version from PR1393 with Cluster plot fixes.
         * https://github.com/mwaskom/seaborn/pull/1393
         * https://github.com/mwaskom/seaborn/blob/fb1f87e800e69ba2e9309f922f9dac470e3a6c78/seaborn/matrix.py
+        * This may break for versions of Seaborn other than 0.9.0 (tested version)
     * Currently only set for single colourmap choice, should set as dict.
     * Clustermap methods from:
         * https://stackoverflow.com/questions/27988846/how-to-express-classes-on-the-axis-of-a-heatmap-in-seaborn
@@ -288,7 +320,23 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
     To do
     -----
     - Improved dim handling, maybe use :py:func:`epsproc.util.matEdimList()` (and related functions) to avoid hard-coding multiple cases here.
+        - Partially implemented in dimMap.
+        - Currently throws an error for None in symmetry plotting cases for unrecognised dataType. TO FIX!
+           - 14/01/21 fixed, but possibly introduced other issues!
+        - Consider consolidating mapping styles further, and keeping a global list of parent and child dims? Or otherwise pulling from stacked Xarray?
     - Improved handling of sets of polarization geometries (angles).
+    - CONSOLIDATE stacked/unstacked dim handling.  At the moment some functions use stacked, some unstacked, which is leading to annoying bugs.
+
+    History
+    -------
+
+    * 14/01/21 - Fixed bug in handling nested dims for unknown datatypes (hopefully), specifically for Sym dims.
+    * 27/02/20 - handling for MultiIndex cases for Wigner 3j type data (function of (l,lp,L,m,mp,M))
+        * Fixed issue with conversion to Pandas table - should now handle dims better.
+        * Improved multidim dropna() using Xarray version (previously used Pandas version).
+        * Added xDim as dict mapping capability.
+    * 05/12/19 - debug
+    * 29/11/19 - initial version.
 
     Examples
     --------
@@ -297,7 +345,7 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
     """
     # Local/deferred import to avoid circular import issues at module level.
     # TO DO: Should fix with better __init__!
-    from epsproc.util import matEleSelector, matEdimList, BLMdimList
+    from epsproc.util import matEleSelector, matEdimList, BLMdimList, dataTypesList, multiDimXrToPD
 
     # Set Seaborn style
     # TO DO: should pass args here or set globally.
@@ -319,9 +367,10 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
     daPlot.attrs = data.attrs
 #     daPlot = data
 
-    # Set filename if missing
-    if 'file' not in daPlot.attrs:
-        daPlot.attrs['file'] = '(No filename)'
+    # Set dataType if missing - NOTE additionally set at end to avoid accidentally dropping this.
+    if 'dataType' not in daPlot.attrs:
+        daPlot.attrs['dataType'] = '(No dataType)'
+        print(f"Set dataType {daPlot.attrs['dataType']}")
 
     # Use SF (scale factor)
     # Write to data.values to make sure attribs are maintained.
@@ -337,10 +386,24 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
                                               # However, does work without this for xr.where() - supports complex comparison.
 
     # Get full dim list
-    if daPlot.attrs['dataType'] is 'matE':
-        dimMap = matEdimList(sType = 'sDict')
+    # Manual
+    # if daPlot.attrs['dataType'] is 'matE':
+    #     dimMap = matEdimList(sType = 'sDict')
+    # else:
+    #     dimMap = BLMdimList(sType = 'sDict')
+
+    # Using util.dataTypesList() - THIS IS NOT yet used in the main plotting routine.
+    # ACTUALLY - is used for Sym plotting case, may throw an error for None case.
+    # See lines ~630, 696
+    dataTypes = dataTypesList()
+    if daPlot.attrs['dataType'] in dataTypes:
+        dimMap = dataTypes[daPlot.attrs['dataType']]['dims']
     else:
-        dimMap = BLMdimList(sType = 'sDict')
+        # dimMap = None  # This is painful and gives errors for arb data types symmetry cases
+        # dimMap = list(daPlot.dims)  # This is OK, but won't get nested dims.
+        dimMap = {item:(list(daPlot.get_index(item).names) if len(daPlot.get_index(item).names)>1 else []) for item in daPlot.dims }  # Return subdims or empty list in dict format
+
+
 
     # Eulers >>> Labels
     if eulerGroup and ('Euler' in daPlot.dims):
@@ -374,12 +437,20 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
 
     # Sum & threshold
     if sumDims is not None:
-        daPlot = daPlot.sum(sumDims).squeeze()
+        daPlot = daPlot.sum(sumDims) #.squeeze()  # Set squeeze condition below.
         daPlot.attrs = data.attrs  # Reset attribs
 
+    # If xDim is stacked, check it exists and create it if not.
+    # TOTAL FUDGE HERE - this may be problematic due to unstack() UGH.
+    # HOWEVER, without this xDim must exist in passed daPlot array, otherwise later code may fail.
+    if type(xDim) == dict:
+        dimCheck = any([item in xDim for item in daPlot.dims])
+        if not dimCheck:
+            daPlot = daPlot.unstack().stack(xDim)
+
     # Threshold on abs() value before setting type, otherwise all terms will appear for some cases (e.g. phase plot)
     # daPlot = matEleSelector(daPlot, thres=thres, inds = selDims, dims = xDim) # , sq = True)  # Squeeze may cause issues here if a singleton dim is used for xDim.
-    daPlot = matEleSelector(daPlot, thres=thres, dims = xDim) # Version without selection, now incorporated above.
+    daPlot = matEleSelector(daPlot, thres=thres, dims = xDim, drop = False, sq = squeeze) # Version without selection, now incorporated above.
     daPlot = plotTypeSelector(daPlot, pType = pType, axisUW = xDim)
 
     # daPlot = ep.util.matEleSelector(daPlot, thres=thres, inds = selDims, dims = 'Eke', sq = True)
@@ -387,6 +458,17 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
     if logFlag:
         daPlot.values = daPlot.pipe(np.log10)
 
+#*** Fix any missing attrs - set this LAST otherwise may be accidentally overwritten above.
+    # Set filename if missing
+    if 'file' not in daPlot.attrs:
+        daPlot.attrs['file'] = '(No filename)'
+
+    # Set dataType if missing
+    if 'dataType' not in daPlot.attrs:
+        daPlot.attrs['dataType'] = '(No dataType)'
+        print(f"Set dataType {daPlot.attrs['dataType']}")
+
+
 #*** Plotting routines
     # Rough code for xr plotter.
     if backend is 'xr':
@@ -411,16 +493,69 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
             lutSym = dict(zip(map(str, np.unique(symList)), palSym))
         else:
             symFlag = False
+            symList = []
+
+        # 14/01/20 Set catch-all defaults for dims with shared cmaps - UGLY, partially implemented below, but still have lots of special cases
+        # 01/02/21 Added set check with xDim to ensure this works for mixed (l,K) cases, with one as xDim (otherwise can miss labels).
+        lDims = list(set(['l', 'K', 'lp', 'L'])-set(xDim))
+        mDims = ['m', 'mp', 'mu', 'Q', 'S', 'mu0', 'Lambda', 'M']
 
         # Set unified cmap for (m,mu)
         # Switch for matE or BLM data type.
 #         if ('m' in daPlot.dims) and ('mu' in daPlot.dims):
+
+        # Set a default case to allow for arb. dim names
+        # TODO: fix this to select based on max in passed QNs.
+        mList = np.arange(-mMax, mMax+1)
+
+        # Set unstaked (full) dim list
+        dimUS = daPlot.unstack().dims
+
+        # Set l (or equivalent) cmap here, ensures ordering over over dims (ordered list returned from daPlot Xarray).
+        # NOTE - set crude default case for max = 10.
+        # NOTE - assumes only one of these dims is present (or will overwrite)
+        # Added 28/09/20
+        # lList = []
+        lList = np.arange(0, 10, dtype=np.int64)  # Set default case. UGLY.
+        # lList = daPlot[xDim].pipe(np.unique)        # Should be able to default to first xDim passed?
+        lListTemp = []
+        for dim in dimUS:
+            # if dim in ['l', 'K', 'lp', 'L']:
+            if dim in lDims:
+                # lList = daPlot[dim].pipe(np.unique)  # This fails for multiple l-types with different values
+                lListTemp.extend(daPlot[dim].pipe(np.unique))  # Stack over all matching dims then recheck for unique values.
+
+        if lListTemp:
+            lList = np.unique(lListTemp)
+
+        # if not lList.any():
+        #     lList = lListDefault
+
+
+        palL = sns.light_palette("green", n_colors=lList.size)
+        lutL = dict(zip(map(str, lList), palL))
+
+
         if 'LM' in daPlot.dims:
-            mList = np.unique(daPlot.LM.m)
+            try:
+                mList = np.unique(daPlot.LM.m)   # For Wigner 3j functions, allow for m or M
+            except AttributeError as e:
+                pass
+                # if e != "'DataArray' object has no attribute 'm'":
+                #     raise
+            try:
+                mList = np.unique(daPlot.LM.M)
+            except AttributeError as e:
+                # if e != "'DataArray' object has no attribute 'M'":
+                #     raise
+                pass
+
         if 'BLM' in daPlot.dims:
             mList = np.unique(daPlot.BLM.m)
+
         if 'ADM' in daPlot.dims:
             mList = np.unique([daPlot.ADM.Q, daPlot.ADM.S])
+
         mColours = mList.size
 
         if mColours < 3:  # Minimal value to ensure mu mapped - may be better to generate without ref here?
@@ -430,30 +565,97 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
         palM = sns.color_palette("coolwarm", mColours)
         lutM = dict(zip(map(str, mList), palM))
 
+# 12/03/20 - moving this to separate conversion function, since tabulation is handy.
+        # Note thres=None set here to avoid putting holes in the pd data. This is a temp workaround!
+        daPlotpd, daPlot = multiDimXrToPD(daPlot, colDims = xDim, rowDims = plotDims, thres = thres, dropna = True, fillna = fillna, squeeze = squeeze, colRound = labelRound)
 
-        # Convert to Pandas 2D array, stacked along plotDims - these will be used for colour bar.
-        # TO DO: fix hard-coded axis number for dropna()
-
-        # Check plotDims exist, otherwise may throw errors with defaults
-        plotDimsRed = []
-        # for dim in daPlot.unstack().dims:
-        #     if dim in plotDims:
+        # # Convert to Pandas 2D array, stacked along plotDims - these will be used for colour bar.
+        # # TO DO: fix hard-coded axis number for dropna()
+        # # 27/02/20: switched to use xr.dropna(dim = 'plotDim', how = 'all') instead of pd.dropna.  This should be more robust... hopefully won't break old code.
+        # #           Also added stack(xDim = xDim) to allow for multilevel X plotting.
+        # # 11/03/20: added auto setting for plotDims, based on sets, plus checks for missing dims.
+        #
+        # # Work-around for dict case - get list of unstacked dims for comparison
+        # if type(xDim) == dict:
+        #     xDimList = list(xDim.items())[0][1]
+        # else:
+        #     xDimList = xDim
+        #
+        # if plotDims is None:
+        #     plotDims = list(set(dimUS) - set(xDimList))   # Use set arithmetic to get items
+        #     plotDims.sort()                      # Set sort to return alphebetical list.
+        #
+        #
+        # # Check plotDims exist, otherwise may throw errors with defaults
+        # plotDimsRed = []
+        # # for dim in daPlot.unstack().dims:
+        # #     if dim in plotDims:
+        # #         plotDimsRed.append(dim)
+        # for dim in plotDims:
+        #     if dim in dimUS:
         #         plotDimsRed.append(dim)
-        for dim in plotDims:
-            if dim in daPlot.unstack().dims:
-                plotDimsRed.append(dim)
+        #
+        # # Additional check for any missing dims
+        # # Check # of dims and correct for any additional/skipped dims
+        # # Bit ugly - should be integrated with above code
+        # if (len(xDimList) + len(plotDimsRed)) != len(dimUS):
+        #     for dim in dimUS:
+        #         if not (dim in xDimList) and not (dim in plotDimsRed):
+        #             plotDimsRed.append(dim)
+        #
+        #             if verbose:
+        #                 print(f'Adding {dim} to plotting dim list.')
+        #
+        #
+        # # Restack for plotting, and drop singleton dimensions if desired.
+        # daPlot = daPlot.unstack().stack(plotDim = plotDimsRed).dropna(dim = 'plotDim', how = 'all')
+        #
+        # # Restack xDim in cases where it is a MultiIndex
+        # if type(xDim) == dict:
+        #     daPlot = daPlot.stack(xDim)
+        #
+        #
+        # if squeeze:
+        #     # daPlotpd = daPlot.unstack().stack(plotDim = plotDimsRed).squeeze().to_pandas().dropna(axis = 1).T
+        #     # daPlotpd = daPlot.unstack().stack(plotDim = plotDimsRed).dropna(dim = 'plotDim', how = 'all').squeeze().to_pandas().T
+        #     daPlotpd = daPlot.squeeze().to_pandas()
+        #
+        # else:
+        #     # daPlotpd = daPlot.unstack().stack(plotDim = plotDimsRed).to_pandas().dropna(axis = 1).T
+        #     # daPlotpd = daPlot.unstack().stack(plotDim = plotDimsRed).dropna(dim = 'plotDim', how = 'all').to_pandas().T
+        #     daPlotpd = daPlot.to_pandas()
+        #
+        # # Transpose Pandas table if necessary - xDim must be columns
+        # if type(xDim) != dict:
+        #     if xDim not in daPlotpd.columns.names:
+        #         daPlotpd = daPlotpd.T
 
-        # Restack for plotting, and drop singleton dimensions if desired.
-        if squeeze:
-            daPlotpd = daPlot.unstack().stack(plotDim = plotDimsRed).squeeze().to_pandas().dropna(axis = 1).T
-        else:
-            daPlotpd = daPlot.unstack().stack(plotDim = plotDimsRed).to_pandas().dropna(axis = 1).T
+
+        # For dictionary case, check items for each key are in column names.
+        # THIS CODE IS HORRIBLE - should be a neater way to do this.
+        # TODO: fix case for some levels missing, at the moment assumes any present is OK.
+        # TODO: test for single vs. MultiIndex case - columns.names vs. columns.name?
+        # else:
+        #     for key in xDim:
+        #         dimList = xDim[key]
+        #         check = [item in daPlotpd.columns.names for item in dimList]
+        #         if not any(check):
+        #             daPlotpd = daPlotpd.T
+# --- END pandas conversion.
+
+        # For Wigner3j datatypes, still get some illegal values creeping through - now try removing again...
+        # if daPlot.dataType == 'Wigner3j':
+        #     daPlotpd = daPlotpd.dropna(axis = 1, how = 'all')
+        # NOW set in multiDimXrToPD, with dropna = True (default)
 
         # Set multi-index indicies & colour mapping
         cList = []
         legendList = []
         for dim in daPlotpd.index.names:
-            Labels = daPlotpd.index.get_level_values(dim) #.astype('str')
+            # 28/09/20: added .unique().sort_values(), should clean up legend a little bit?
+            # EDIT: actually added only for legendList.append() stuff below.
+            # Q: this should be OK, but may mess up multidim labels...?
+            Labels = daPlotpd.index.get_level_values(dim)  # .unique().sort_values() #.astype('str')
 
             if verbose:
                 print(dim)
@@ -461,13 +663,19 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
 
             # For (l,m,mu) set colour scales as linear (l), and diverging (m,mu)
             # May want to modify (m,mu) mappings to be shared?
-            if dim in ['l','K']:
-                pal = sns.light_palette("green", n_colors=Labels.unique().size)
-                lut = dict(zip(map(str, Labels.unique()), pal))
+            # 28/09/20: revisiting to fix ordering over multiple states.
+            #   Should be fixed by setting master list above, as per M state settings, since that takes ordered list over full dataset.
+            #   (Otherwise l cmap may be split by sym or other dim ordering)
+            # if dim in ['l', 'K', 'lp', 'L']:
+            if dim in lDims:
+                # pal = sns.light_palette("green", n_colors=Labels.unique().size)
+                # lut = dict(zip(map(str, Labels.unique()), pal))
+                pal = palL
+                lut = lutL
                 cList.append(pd.Series(Labels.astype('str'), index=daPlotpd.index).map(lut))  # Mapping colours to rows
-                legendList.append((Labels.unique(), lut))
+                legendList.append((Labels.unique().sort_values(), lut))
 
-            elif dim in ['m', 'mu', 'Q', 'S']:
+            elif dim in mDims:
 #                 pal = sns.diverging_palette(220, 20, n=Labels.unique().size)
 #                 lut = dict(zip(map(str, Labels.unique().sort_values()), pal))  # Note .sort_values() required here.
                 pal = palM
@@ -478,13 +686,19 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
             # If mapping symmetries, use flattened list set above for colour mapping
             # This will keep things consistent over all sym sub-labels
             # NOTE - symFlag setting allows for case when dimMap['Sym'] is missing (will throw an error otherwise)
+            # 24/06/20 - removed "and" here - throws error for unmapped types. Nope, reinstated - causes other issues!
+            # 14/01/21 - hopefully fixed by using symList, and setting empty default.
             elif symFlag and (dim in dimMap['Sym']):
+            # elif symFlag and (dim in symList):
+            # elif symFlag or (dim in symList):
 #                 pal = sns.color_palette("hls", np.unique(symList).size)
 #                 lut = dict(zip(map(str, np.unique(symList)), pal))
+
                 pal = palSym
                 lut = lutSym
                 cList.append(pd.Series(Labels.astype('str'), index=daPlotpd.index).map(lut))  # Mapping colours to rows
-                legendList.append((Labels.unique(), lut))
+                legendList.append((Labels.unique().sort_values(), lut))
+                # pass
 
             # For other dims, set to categorical mapping.
             # Should check for shared dims here, e.g. Syms, for mapping...?
@@ -497,7 +711,7 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
 #                 pal = sns.light_palette("blue", n_colors=Labels.unique().size)
                 lut = dict(zip(map(str, Labels.unique()), pal))
                 cList.append(pd.Series(Labels.astype('str'), index=daPlotpd.index).map(lut))  # Mapping colours to rows
-                legendList.append((Labels.unique(), lut))
+                legendList.append((Labels.unique().sort_values(), lut))
 
             # For legend, keep also Labels.unique, lut, pairs
 #             legendList.append((Labels.unique(), lut, dim))  # Include dim... redundant if Labels is pd.index with name
@@ -509,6 +723,10 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
         for c in cList[1:]:
             colors = colors.join(pd.DataFrame(c))
 
+        # Rounding for long labels on xDim
+        # if daPlotpd.columns.name in ['Eke','Ehv']:
+        # NOW SET IN multiDimXrToPD()
+
         # *** Plot with modified clustermap code.
         g = snsMatMod.clustermap(daPlotpd,
                   # cmap="vlag", center = 0,
@@ -527,25 +745,76 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
         # Using this method it's only possible to set two sets of legends, split these based on n here.
         # Method from https://stackoverflow.com/questions/27988846/how-to-express-classes-on-the-axis-of-a-heatmap-in-seaborn
         # TO DO: For further control over legend layout may need to add dummy items: https://stackoverflow.com/questions/34212241/control-the-number-of-rows-within-a-legend
+
+        # 14/01/21: testing with preset dims & conditionals
+        # TO DO: Should be able to just use lutM list and plot global min-max? Currently may have issues with different sized dims?
+        # 03/05/21 - testing label rounding here, for float quantities. This fails either for tuple overwrite or on lookup later, ugh. HORRIBLE SHIT.
+        #           NOW FORCED WITH enumerate(), but it's still pretty UGLY.  Seems OK in testing, working for legend labels only.
+        # TODO: fix with Pandas methods? Should be able to fix for ledgend & axis labels?
+        # NOTE: this is set for column labels with multiDimXrToPD() above, should also add index rounding there?
+        mDimLabel = None
         for n, item in enumerate(legendList):
             # To avoid issues with long floats, round to 3dp
             # TODO: fix this, currently fails for Euler angles case?
-            # if item[0].dtype == np.float:
-            #     item[0] = np.round(item[0],3)
+            # 26/09/20: added name check here, hopefully this will fix Euler case...TBC...
+            # 03/05/21: retested this... failing for all tried variations, due to either tuple immutability and/or lookups later if values changed.  THIS IS HORRIBLE CODE.
+            # Also added above for xDim labels.
+            if (item[0].dtype == np.float) or (item[0].name in ['Eke','Ehv','t','Euler']):
+                labelsR = np.round(item[0],3).astype('str')
+            else:
+                labelsR = item[0].astype('str')
 
-            for label in item[0].astype('str'):
+            # for label in item[0].astype('str'):
+            for ln, label in enumerate(item[0].astype('str')):
+            # for label in labels:
+
+                # 03/05/21 - testing label rounding here, this fails on lookup later, ugh.
+                # if isinstance(label, np.float):
+                #     labelR = np.round(label,3).astype('str')
+                # else:
+                #     labelR = label  #.astype('str')
+                labelR = labelsR[ln]
+
+                if verbose:
+                    print(f"Legend entry: {label} for {item}")
 #                 label = string(label)
 #                 if n%2:
-                if item[0].name in ['l','m','K','Q']:
+
+                # 1st legend for l,m,mu
+#                 if item[0].name in ['l','m','K','Q','Lambda']:
+#                     g.ax_col_dendrogram.bar(0, 0, color=item[1][label],label=label, linewidth=0)
+#                 elif item[0].name in ['mu','S','mu0']:  # Skip to avoid repetition, assuming m or Q already plotted on same colour scale. TODO: add conditionals here?
+#                     pass
+# #                 elif symFlag and (item[0].name is 'Cont'):
+# #                     g.ax_row_dendrogram.bar(0, 0, color=item[1][label],label=label, linewidth=0)
+#                 elif symFlag and (item[0].name in dimMap['Sym']):  # Skip symmetries
+#                 # elif symFlag and (item[0].name in symList):  # Skip symmetries
+#                     pass
+
+                # 14/01/21: testing with preset dims & conditionals
+                if item[0].name in lDims:
                     g.ax_col_dendrogram.bar(0, 0, color=item[1][label],label=label, linewidth=0)
-                elif item[0].name in ['mu','S']:  # Skip to avoid repetition, assuming m or Q already plotted on same colour scale.
-                    pass
+                    # g.ax_col_dendrogram.bar(0, 0, color=item[1][label],label=labelR, linewidth=0)
+                elif item[0].name in mDims:
+                    if mDimLabel is None:  # Set first found mDim as source of labels - may cause issues in some cases? Should be able to just use lutM list?
+                        mDimLabel = item[0].name
+                    if item[0].name == mDimLabel:
+                        g.ax_col_dendrogram.bar(0, 0, color=item[1][label],label=f"({label})", linewidth=0)
+                        # g.ax_col_dendrogram.bar(0, 0, color=item[1][label],label=f"({labelR})", linewidth=0)
+
+                # elif item[0].name in ['mu','S','mu0']:  # Skip to avoid repetition, assuming m or Q already plotted on same colour scale. TODO: add conditionals here?
+                #     pass
 #                 elif symFlag and (item[0].name is 'Cont'):
 #                     g.ax_row_dendrogram.bar(0, 0, color=item[1][label],label=label, linewidth=0)
                 elif symFlag and (item[0].name in dimMap['Sym']):  # Skip symmetries
+                # elif symFlag and (item[0].name in symList):  # Skip symmetries
                     pass
+
+                # 2nd legend for other dims.
                 else:
-                    g.ax_row_dendrogram.bar(0, 0, color=item[1][label],label=label, linewidth=0)
+                    # g.ax_row_dendrogram.bar(0, 0, color=item[1][label],label=label, linewidth=0)  # Label with value only
+                    # g.ax_row_dendrogram.bar(0, 0, color=item[1][label],label=f"{item[0].name}: {label}", linewidth=0)  # Label with category too.
+                    g.ax_row_dendrogram.bar(0, 0, color=item[1][label],label=f"{item[0].name}: {labelR}", linewidth=0)  # Label with category too.
 
         # Add symmetry labels separately from master list to avoid repetition
         if symFlag:
@@ -567,7 +836,19 @@ def lmPlot(data, pType = 'a', thres = 1e-2, thresType = 'abs', SFflag = True, lo
         # Additional anootations etc.
         # Plot titles: https://stackoverflow.com/questions/49254337/how-do-i-add-a-title-to-a-seaborn-clustermap
         # g.fig.suptitle(f"{daPlot.attrs['file']}, pType={pType}, thres={thres}")  # Full figure title
-        g.ax_heatmap.set_title(f"{daPlot.attrs['file']}, plot type = {daPlot.attrs['pTypeDetails']['Type']}, threshold = {np.round(thres, 2)}, inc. cross-section {SFflag}, log10 {logFlag}")  # Title heatmap subplot
+        if thres is not None:
+            thresStr = np.round(thres, 2)
+        else:
+            thresStr = 'None'
+
+        # Set main plot title if not passed.
+        if titleString is None:
+            if hasattr(daPlot,'jobLabel'):
+                titleString = daPlot.attrs['jobLabel']
+            else:
+                titleString = daPlot.attrs['file']
+
+        g.ax_heatmap.set_title(f"{titleString}\n Plot type = {daPlot.attrs['pTypeDetails']['Type']}, threshold = {thresStr}\n Inc. cross-section {SFflag}, log10 {logFlag}")  # Title heatmap subplot
 
         # sns.reset_orig()  # Reset gloabl plot params - leaves rc settings, also triggers Matplotlib errors
         sns.reset_defaults()
